/* 
 * Purpose: Plot billing collection rate against kWH consumed (Figure 6)
 * Data source: individual billing data from North Bihar, Oct 2017-June 2018
 */


pause on
set more off
set varabbrev off
set matsize 10000
graph drop _all
cap log close
set seed 082196
clear

*** file paths

if "`c(os)'" == "MacOSX" {
*	global ROOT ""
	cd "$ROOT/Code"
	local OUT_DATA "$ROOT/Data"
	local GRAPHS "$ROOT"
}

*** graph fonts
set scheme s2mono
graph set window fontface "Times New Roman"

*** switches
local use_kwh_averages = 1
// cleaning and graph parameters
local max_unitsconsumed = 1000 // in kWh, for plots by kWh
local round_nearest = 50
local max_logunitsconsumed = 4 // in log base 10 of kWh, for log kwh plots
local round_nearest_log = 0.25


if `use_kwh_averages' == 1 {

	* save non-rounded dataset of units consumed (for overlaying histogram later)
	use "`OUT_DATA'/billing_collection_oct17_jun18.dta", clear
	keep unitsconsumed logunitsconsumed
	tempfile tempunits
	save "`tempunits'"
	
	* load in dataset
	use "`OUT_DATA'/billing_collection_oct17_jun18.dta", clear
		
	* IMPORTANT: totalcollection can include repayment for previous debts, while assessmentwithdps is a monthly measure.
	* So collection rates (after seasonality) should not exceed 1. If they do, people are overpaying.
	* To deal with this, we truncate collection amount at (105% of) assessmentdps amount.
	replace totalcollection = assessmentwithdps*1.05 if collection_rate > 1.05 & !missing(collection_rate)
	replace collection_rate = 1.05 if collection_rate > 1.05 & !missing(collection_rate)
	
	* generate payment rate at some rounded interval
	replace unitsconsumed = floor(unitsconsumed/`round_nearest')*`round_nearest'
	replace logunitsconsumed = floor(logunitsconsumed/`round_nearest_log')*`round_nearest_log'
	* gen count var
	gen count = 1	
	
	*** graph collection rate and contribution to losses by kWh
	graph drop _all
	foreach xvar in "logunitsconsumed" {	
		preserve
			collapse (sum) assessmentwithdps totalcollection count, by(`xvar')
			gen collection_rate = totalcollection/assessmentwithdps
			
			* save total billed and collected to get share of losses
			keep if `xvar' <= `max_`xvar''
			summ assessmentwithdps
			local sum_assessmentwithdps = r(sum)
			summ totalcollection
			local sum_totalcollection = r(sum)
			gen share_of_losses = (assessmentwithdps - totalcollection) / (`sum_assessmentwithdps' - `sum_totalcollection')
			* gen cumulative share of losses adjust highest category (for unitsconsumed)
			sort `xvar'
			gen sum_share_of_losses = sum(share_of_losses)
			if "`xvar'" == "unitsconsumed" {
				replace share_of_losses = 1 - sum_share_of_losses[_n-1] if unitsconsumed == `max_unitsconsumed'
			}
			
			* merge in data for histogram
			merge 1:m `xvar' using "`tempunits'"
			
			* graph
			local xdetails ""
			if "`xvar'" == "unitsconsumed" {
				keep if unitsconsumed <= `max_unitsconsumed' 
				local xdetails "xlabel(0(200)`max_unitsconsumed') xscale(range(0 `max_unitsconsumed'))"
				local histdetails ""
			} 
			else if "`xvar'" == "logunitsconsumed" {
				keep if logunitsconsumed >= 0 & logunitsconsumed <= `max_logunitsconsumed'
				local xdetails "xlabel(0(1)`max_logunitsconsumed', valuelabel) xscale(range(0 `max_logunitsconsumed'))"
				local histdetails "start(0)"
			}
			sort `xvar'
			graph twoway (hist `xvar', frac `histdetails' yaxis(2) color(%30) ytitle("", axis(2))) ///
				(line collection_rate `xvar', c(l) yaxis(1)) ///
				(line sum_share_of_losses `xvar', c(l) yaxis(1)) , ///
				name(g`xvar') `xdetails' ylabel(0(.2)1) yscale(range(0 1)) graphregion(color(white)) bgcolor(white) plotregion(color(white)) ///
				xtitle("Average Monthly Consumption (kWh)") ///
				legend(label(1 "Fraction of Customers (left scale)") label(2 "Bill Payment Rate (right scale)") label(3 "Cumulative Share of Collection Losses (right scale)") ///
					order(2 3 1) cols(1) region(lcolor(white)) ) ///
				
			graph export "`GRAPHS'/collection_rate_share_losses_by_`xvar'.png", replace

		restore
	}
	
}











