
#Description: Import tariff schedule data from xlsx and convert to by-kwh data

rm(list = ls())
dev.off()

library(dplyr)
library(ggplot2)
library("readxl")

############################# UPDATE ROOT HERE BEFORE RUNNING
ROOT <- "/Users/vishannigam/Downloads/Replication Files"
#############################################################


INPUT <- paste(ROOT, "/Data", sep = '')
setwd(paste(ROOT, "/Code", sep = ''))


###

# import data
tariff_raw <- read_excel(paste(INPUT, "/tariff_schedules_combined.xlsx", sep = ''), sheet = "Data")

# create 1000 observations (1/kwh)
tariff_kwh <- tariff_raw[rep(1:nrow(tariff_raw),each=1000),]

# label each kwh
tariff_kwh <- tariff_kwh %>% 
  group_by(discom) %>%
  mutate(kwh = row_number() %% 1000) %>% 
  mutate(kwh = ifelse(kwh == 0, 1000, kwh)) %>%
  ungroup()

# function to generate block variable for each discom/kwh
all_block <- function(df) {
  # declare block matrix
  block_matrix <- NULL

  for (i in 1:9) {
    # for each potential block 1-9, get current and next block minimum
    min_col <- paste("block_", i, "_min", sep = '')
    min_col_next <- paste("block_", i+1, "_min", sep = '')
    # if in block i, return i, otherwise return 0. Returns NA if no block i exists.
    in_block <- if_else(df$kwh >= df[min_col] & (df$kwh < df[min_col_next] | is.na(df[min_col_next])),
            i, as.integer(0))
    # update block matrix with vector that =i if in column i, 0 if before i, NA if no block i exists
    block_matrix = cbind(block_matrix,in_block)
  }
  
  # take row sum (i.e. create vector of block id)
  which_block <- rowSums(block_matrix, na.rm = TRUE)
  return(which_block)
}

# generate block variable, marginal cost, and fixed cost variables
tariff_kwh <- tariff_kwh %>%
  mutate(block = all_block(.)) %>% # get block variable
  group_by(discom, block) %>% # pull discom rate from correct block column
  mutate(mc = eval(parse(text = paste("block_", block, "_rate", sep = '')))) %>%
  ungroup() %>%
  mutate(fc = if_else(block == 1, fixed_charge_1, fixed_charge_rest)) # get correct FC

# 1) adjust marginal cost (for total cost computation) in case where
# mc on block 1 units increase when total consumption is in or above block 2, 
# 2) get tc_adj = fc + running sum of mc_adj (using adjusted MC) and tc = fc + sum of mc (not adjusted)
# 3) create one tc variable
tariff_kwh <- tariff_kwh %>%
  group_by(discom) %>% # for each discom
  mutate(tempvar = mc[which(block == 2)[1]]) %>% # save the block 2 MC
  ungroup() %>%
  # if in block 1 and condition met, store block 2 MC, otherwise store original MC
  mutate(mc_adj = ifelse(block == 1 & higher_rate_all_units, tempvar, mc)) %>%
  group_by(discom) %>% # again for each discom
  mutate(tc = fc + cumsum(mc)) %>% # get total cost for block 1
  mutate(tc_adj = fc + cumsum(mc_adj)) %>% # get total cost that applies to other blocks
  mutate(tc = ifelse(block == 1, tc, tc_adj)) %>%
  ungroup()

# 1) adjust tc for monthly minimum
# 2) adjust mc for monthly minimum
# 3) re-calculate "true" mc as difference in tc (including the big jumps from revising MC on early units)
# 4) compute ac
tariff_kwh <- tariff_kwh %>%
  mutate(tc = pmax(tc, monthly_min)) %>% # if total cost is below monthly minimum, replace w the monthly min
  mutate(mc = ifelse(tc == monthly_min, 0, mc)) %>% # if now-adjusted tc still = monthly min, marginal cost is 0
  group_by(discom) %>%
  mutate(mc_true = tc - lag(tc, default = tc[1])) %>% # calculate "true" mc
  ungroup() %>%
  mutate(ac = tc / kwh) # get average cost / kwh

# make T&D loss a proportion from 0-1
# then adjust purchase cost by dividing by (1-T&D)
tariff_kwh <- tariff_kwh %>% 
  mutate(td_loss = td_loss/100) %>%
  mutate(purchase_cost_adj = (purchase_cost / (1 - td_loss)))

# only keep certain variables
tariff_kwh <- tariff_kwh %>% select(country, discom, country_pop, discom_weight,
                                    income_range, kwh, block, fc, mc, mc_true, ac, tc,
                                    has_lifeline, avg_hh_cons, purchase_cost, purchase_cost_adj,
                                    purchase_cost_name, usd_exchange_rate, td_loss)

# adjust to usd
tariff_kwh_usd <- tariff_kwh %>%
  mutate(fc_usd = fc*usd_exchange_rate,
         mc_usd = mc*usd_exchange_rate,
         mc_true_usd = mc_true*usd_exchange_rate,
         ac_usd = ac*usd_exchange_rate,
         tc_usd = tc*usd_exchange_rate,
         purchase_cost_usd = purchase_cost*usd_exchange_rate,
         purchase_cost_adj_usd = purchase_cost_adj*usd_exchange_rate)

save(tariff_kwh_usd, file = paste(INPUT, "/tariffs_by_kwh.RData", sep = ''))


