#Description: Use cleaned by-kwh tariff schedule to plot rate schedules
# and make a table of plots by country

rm(list = ls())

library(dplyr)
library(ggplot2)
library("readxl")
library(knitr)
library(kableExtra)
library(extrafont)
library(lazyeval)
library(stargazer)
options(knitr.table.format = "latex")

# font size for ggplot
theme_set(theme_grey(base_size = 18)) 

############################# UPDATE ROOT HERE BEFORE RUNNING
ROOT <- "/Users/vishannigam/Downloads/Replication Files"
#############################################################

INPUT <- paste(ROOT, "/Data", sep = '')
OUTPUT <- ROOT
setwd(paste(ROOT, "/Code", sep = ''))



# load data
load(paste(INPUT, "/tariffs_by_kwh.RData", sep = ''))


####### BY INCOME CATEGORY
# get weighted mean (by discom_weight within country, then UNWEIGHTED across countries)
by_country_w <- tariff_kwh_usd %>% 
  group_by(country, country_pop, income_range, kwh) %>%
  summarize(mc_avg = weighted.mean(mc_usd, discom_weight), 
            ac_avg = weighted.mean(ac_usd, discom_weight),
            pc_avg = weighted.mean(purchase_cost_usd, discom_weight),
            td_loss_avg = weighted.mean(td_loss, discom_weight),
            pc_adj_avg = weighted.mean(purchase_cost_adj_usd, discom_weight),
            avg_hh_cons_avg = weighted.mean(avg_hh_cons, discom_weight))

by_income_w <- by_country_w %>%  # note that with weights, we collapse COUNTRY-level data to income group
  group_by(income_range, kwh) %>%
  summarize(mc_avg = mean(mc_avg), 
            ac_avg = mean(ac_avg),
            pc_avg = mean(pc_avg),
            td_loss_avg = mean(td_loss_avg),
            pc_adj_avg = mean(pc_adj_avg),
            avg_hh_cons_avg = mean(avg_hh_cons_avg)) %>%
  mutate(ac_less_pc = ac_avg - pc_adj_avg)

# we are left, for each kwh and income group, with the variables:
# 1) mc_avg = marginal price per kwh, averaged across discoms
# 2) ac_avg = average price per kwh, averaged across discoms
# 3) pc_avg = average purchase cost per kwh, averaged across discoms
# 4) avg_hh_cons_avg = average household consumption averaged across countries
# 5) ac_less_pc = ac_avg - pc_avg

# plot average mc or ac (mc_avg or ac_avg)
plot_averages <- function(which_weight, which_var){
  
  if(which_weight == TRUE){
    df <- by_income_w
    weight_name <- "w"
  } else {
    df <- by_income_uw
    weight_name <- "uw"
  }
  
  if(which_var == "mc_avg") {
    y_lim <- c(0,0.31)
    var_name <- "Marginal Price"
  } else if(which_var == "ac_avg") {
    y_lim <- c(0,0.31)
    var_name <- "Average Price"
  } else {
    var_name <- "Average Price less Power Purchase Cost"
    y_lim <- c(-.1,.3)
  }
  
  # define data franme for ggplot
  to_graph <- df %>%
    filter(income_range == "High") %>%
    select(kwh, outcome = which_var, avg_hh_cons_avg) %>%
    bind_rows(df %>% filter(income_range == "Upper-middle") %>% 
                select(kwh, outcome = which_var, avg_hh_cons_avg)) %>%
    bind_rows(df %>% filter(income_range == "Lower-middle") %>% 
                select(kwh, outcome = which_var, avg_hh_cons_avg)) %>%
    bind_rows(df %>% filter(income_range == "Low") %>% 
                select(kwh, outcome = which_var, avg_hh_cons_avg))

  # order income ranges for graph
  to_graph$income_range <- factor(to_graph$income_range, levels = c("Low", "Lower-middle", "Upper-middle", "High"))
  # get vertical line values (avg hh consumption for countries in the cohort)
  vline_data <- to_graph %>% group_by(income_range) %>% summarize(mean_cons = mean(avg_hh_cons_avg))

  # plot
  ggplot(to_graph, aes(x = kwh)) + 
    geom_line(aes(y = outcome, linetype = income_range), size = .6) +
    scale_linetype_manual(values=c("longdash", "dashed", "dotted", "F1"), name = "Country Income Range") +
    scale_x_continuous(expand = c(0,0), limits = c(0,1050)) + 
    scale_y_continuous(expand = c(0,0), limits = c(y_lim)) + 
    theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
          panel.background = element_blank(), axis.line = element_line(colour = "black")) +
    labs(y=paste(var_name, " \n(USD/kWh)", sep = ''), x="Electricity Consumption (kWh/month)") +
    theme(legend.position= "bottom", legend.box = "horizontal") +
    theme(legend.key.width = unit(0.6, "in"), legend.key.height = unit(0.3, "in")) +
    theme(legend.title=element_text(size=15), legend.text=element_text(size=10)) +
    geom_vline(data = vline_data, aes(xintercept = mean_cons), size = .6, color = "gray") +
    geom_text(data = vline_data %>% filter(income_range != "Low"), aes(x=mean_cons, label=paste("\n", income_range, sep = ''), y=.27), 
              angle=90, size = 4, color = "gray") +
    geom_text(data = vline_data %>% filter(income_range == "Low"), aes(x=mean_cons, label=paste(income_range,"\n", sep = ''), y=.27), 
              angle=90, size = 4, color = "gray")

  ggsave(paste(OUTPUT,"/",which_var, "_by_income_", weight_name, ".pdf", sep = ''),
         height = 6, width = 9, units = "in")
}

######## In paper submission
plot_averages(TRUE, "mc_avg")




#### Make table:

# inputs to function: appropriate data frame, and grouping variable as string
make_table <- function(group_name, which_weight) {
  # for each country or income group (group_name), only keep
  # kwh observation closest to mean HH consumption
  
  # if-then statements to get correct data frame
  if(which_weight == TRUE){
    weight_name <- "w"
  } else {
    weight_name <- "uw"
  }
  if(group_name == "country"){
    df <- eval(parse(text = paste("by_country_", weight_name, sep = '')))
  } else {
    df <- eval(parse(text = paste("by_income_", weight_name, sep = '')))
  }

  # Note text
  
  output_table <- df %>%
    mutate(avg_cons_dist = abs(kwh - avg_hh_cons_avg)) %>% # compute diff from average
    group_by(group_var = eval(parse(text = group_name))) %>% # group by country or income range
    filter(avg_cons_dist == min(avg_cons_dist)) %>% # only keep obs closest to mean consumption
    ungroup()
  
  output_table <- output_table %>% 
    mutate(td_loss_avg = td_loss_avg * 100) %>% # convert TD to percent
    arrange(kwh) %>% # sort increasing order of consumption
    mutate(unit_profit = 100*(ac_avg - pc_avg)/pc_avg, 
           unit_profit_adj = 100*(ac_avg - pc_adj_avg)/pc_adj_avg)
  
  print(colnames(output_table))
  output_table <- output_table %>%
    select(group_var, kwh, mc_avg, ac_avg, td_loss_avg, pc_avg, unit_profit,
            pc_adj_avg, unit_profit_adj)   # return columns of output table
  
  return(output_table)

}

### output the weighted income range table as CSV
tariffs_by_income_raw <- make_table("income_range", TRUE) %>%
  mutate(price_cost_diff =  ac_avg - pc_adj_avg) %>%
  select(group_var, kwh, ac_avg, pc_avg, pc_adj_avg, price_cost_diff)

write.csv(tariffs_by_income_raw, paste(OUTPUT,"/tariffs_by_income_raw.csv", sep = ''))



